//
//  ViewController.swift
//  CollectionViewDemo
//
//  Created by Kevin McNeish on 5/14/15.
//  Copyright (c) 2015 Oak Leaf Enterprises, Inc. All rights reserved.
//

import UIKit

class ViewController: UICollectionViewController, UICollectionViewDataSource, UICollectionViewDelegateFlowLayout {
    
    var imagesBySection = Dictionary<String, Array<String>>()
    var sectionKeys = Array<String>()
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.imagesBySection = ["ocean": ["BeachRunners", "HookipaSunset", "HookipaSurfer", "KidsWalkingBeach", "OahuEastShore2", "MauiSunset2"],
        "flora": ["BambooForest", "CoffeeShopFlowers", "FlowerCloseup2", "FlowerCloseup3", "FlowerCloseup4", "KauiaRedFlower"],
        "fauna": ["BeachTurtles2", "BreachingWhaleFull", "DogEarCropped", "WhaleTailBlue", "GeckoJelly", "DorsalFin7"]]
        
        self.sectionKeys = Array(self.imagesBySection.keys)
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }

    override func collectionView(collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        
        let key = self.sectionKeys[section]
        return self.imagesBySection[key]!.count
    }
    
    override func numberOfSectionsInCollectionView(collectionView: UICollectionView) -> Int {
        return self.imagesBySection.count
    }
    
    override func collectionView(collectionView: UICollectionView, cellForItemAtIndexPath indexPath: NSIndexPath) -> UICollectionViewCell {
        
        let cell = collectionView.dequeueReusableCellWithReuseIdentifier("ImgCell", forIndexPath: indexPath) as! ImageViewCell
        
        // Set up cell...temp
        let sectionKey = self.sectionKeys[indexPath.section]
        let imageNameArray = self.imagesBySection[sectionKey]!
        cell.imageView.image = UIImage(named: imageNameArray[indexPath.row])
        
        return cell
    }
    
    override func collectionView(collectionView: UICollectionView, viewForSupplementaryElementOfKind kind: String, atIndexPath indexPath: NSIndexPath) -> UICollectionReusableView {
        
        var reusableView: UICollectionReusableView!
        
        if kind == UICollectionElementKindSectionHeader {
            
            let headerView = collectionView.dequeueReusableSupplementaryViewOfKind(UICollectionElementKindSectionHeader, withReuseIdentifier: "HeaderView", forIndexPath: indexPath) as! CollectionHeaderView
            
            let sectionKey = self.sectionKeys[indexPath.section]
            
            headerView.lblHeader.text = sectionKey
            reusableView = headerView
        }
        
        return reusableView
    }

}

