//
//  ViewController.swift
//  DatePickerTimerDemo
//
//  Created by Kevin McNeish on 6/4/15.
//  Copyright (c) 2015 Oak Leaf Enterprises, Inc. All rights reserved.
//

import UIKit
import AVFoundation

class ViewController: UIViewController {
    
    var timer : NSTimer!
    var duration: NSTimeInterval!
    var minutes = 0
    var seconds = 0
    var isTimerPaused = false
    var isTimerStarted = false
    var avSound = AVAudioPlayer()

    @IBOutlet weak var datePicker: UIDatePicker!
    @IBOutlet weak var lblTime: UILabel!
    @IBOutlet weak var btnStartCancel: UIButton!
    @IBOutlet weak var btnPauseResume: UIButton!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view, typically from a nib.
        self.datePicker.countDownDuration = 300
        self.datePicker.setValue(UIColor.whiteColor(), forKey: "textColor")
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }

    @IBAction func startCancel(sender: AnyObject) {
        
        if self.isTimerStarted {
            self.cancelTimer()
        }
        else {
            // Get the # of seconds from the
            // date picker, calc minutes / seconds
            self.duration = self.datePicker.countDownDuration
            self.minutes = Int(floor(duration/60))
            self.seconds = Int(round(Double(duration) - Double(minutes) * 60))
    
            self.startTimer()
       }
    }
    
    func startTimer() {
        // Save the start time in the timer label
        self.updateLabel()
        
        // Hide the date picker, show the time
        self.datePicker.hidden = true
        self.lblTime.hidden = false
        
        // Change the green Start button to a red Cancel
        self.btnStartCancel.setTitle("Cancel", forState: UIControlState.Normal)
        self.btnStartCancel.setTitleColor(UIColor.redColor(), forState: UIControlState.Normal)

        // Enable the pause button
        self.btnPauseResume.enabled = true
        self.isTimerPaused = false
        
        // Start the timer
        self.timer = NSTimer.scheduledTimerWithTimeInterval(1.0, target: self, selector: "countDown", userInfo: nil, repeats: true)

        self.isTimerStarted = true
    }
    
    func cancelTimer() {
        // Stop the timer
        self.timer.invalidate()
        
        // Hide the time, show the date picker
        self.lblTime.hidden = true
        self.datePicker.hidden = false
        
        // Change the red Cancel button to a green Start
        self.btnStartCancel.setTitle("Start", forState: UIControlState.Normal)
        self.btnStartCancel.setTitleColor(UIColor.greenColor(), forState: UIControlState.Normal)
        
        // Disable the pause button
        self.btnPauseResume.enabled = false
        
        self.isTimerStarted = false
    }
    
    @IBAction func pauseResume(sender: AnyObject) {
        if self.isTimerPaused {
            self.startTimer()
            self.btnPauseResume.setTitle("Pause", forState: UIControlState.Normal)
            self.isTimerPaused = false
        }
        else {
            
            self.timer.invalidate()
            self.btnPauseResume.setTitle("Resume", forState: UIControlState.Normal)
            self.isTimerPaused = true
        }
    }
    
    func timerFinished() {
        
        self.cancelTimer()
        
        // Play a sound to indicate timer is finished
        let soundURL = NSBundle.mainBundle().URLForResource("Trill", withExtension: "wav")
        self.avSound = AVAudioPlayer(contentsOfURL: soundURL, error: nil)
		self.avSound.play()
        
        let alertController = UIAlertController(title: "", message: "Timer Done", preferredStyle: .Alert)
        
        
        let OKAction = UIAlertAction(title: "OK", style: .Default) { (action) in
            // ...
        }
        alertController.addAction(OKAction)
        
        self.presentViewController(alertController, animated: true) {
            // ...
        }
    }
    
    func countDown() {
        if self.seconds >= 1 {
            self.seconds--
        }
        else {
            self.minutes--
            self.seconds = 59
        }
        self.updateLabel()
        if self.minutes == 0 && self.seconds == 0 {
            self.timerFinished()
        }
    }
    
    func updateLabel() {
        self.lblTime.text = String(format: "%02d:%02d", minutes, seconds)
    }

}

