//
//  ViewController.swift
//  PageControlDemo
//
//  Created by Kevin McNeish on 5/28/15.
//  Copyright (c) 2015 Oak Leaf Enterprises, Inc. All rights reserved.
//

import UIKit

class ViewController: UIViewController, UIScrollViewDelegate {
    
    @IBOutlet weak var scrollView: UIScrollView!
    @IBOutlet weak var pageControl: UIPageControl!
    var contentList = ["1.png","2.png","3.png","4.png","5.png","6.png"]
    var viewControllers = NSMutableArray()
    var pageControlUsed = false

    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view, typically from a nib.
    }
    
    override func viewDidAppear(animated: Bool) {
        
        // There will be one view controller for each page
        // View controllers are created lazily
        // In the meantime, load the array with placeholders
        // which will be replaced on demand
		let pageCount = self.contentList.count
        var controllers = NSMutableArray()
        for index in 0...contentList.count {
         	controllers.addObject(NSNull())
        }
        self.viewControllers = controllers
        
        // Change the content size of the scrollview to be
        // the current width of the scrollview * the number of pages
        self.scrollView.contentSize = CGSizeMake(self.scrollView.frame.size.width *
            CGFloat(self.contentList.count), self.scrollView.frame.size.height)
        
        self.scrollView.scrollsToTop = false
        
        // Set the number of pages and current page in the page control
        self.pageControl.numberOfPages = pageCount
        self.pageControl.currentPage = 0
        
        // load the first visible page
        // load the next page to avoid flashes when the user starts scrolling
        self.loadScrollViewWithPage(0)
        self.loadScrollViewWithPage(1)
    }
    
    func loadScrollViewWithPage(page: Int) {
        if page < 0 {
            return
        }
        if page >= self.contentList.count {
            return
        }
        
        // If a view controller hasn't been created for this page yet, create one
        var controller: AnyObject = self.viewControllers.objectAtIndex(page)
        if controller as! NSObject == NSNull() {
            let storyboard = UIStoryboard(name: "Main", bundle: nil)
            controller = storyboard.instantiateViewControllerWithIdentifier("MyViewController") as! MyViewController
            self.viewControllers.replaceObjectAtIndex(page, withObject: controller)
        }
        
        // If the controller's view hasn't been added to the scroll view yet, add it
        if let vc = controller as? MyViewController {
            
            if vc.view.superview  == nil {
                var frame = self.scrollView.frame
                frame.origin.x = frame.size.width * CGFloat(page)
                frame.origin.y = frame.origin.y - 20
                vc.view.frame = frame
                vc.numberImage.image = UIImage(named: self.contentList[page])
                self.scrollView.addSubview(vc.view)
            }
        }
        
    }
    
    func scrollViewDidScroll(scrollView: UIScrollView) {
        // We don't want a "feedback loop" between the UIPageControl and the scroll delegate in
        // which a scroll event generated from the user hitting the page control triggers updates from
        // the delegate method. We use a boolean to disable the delegate logic when the page control is used.

        if pageControlUsed {
            // do nothing - the scroll was initiated from the page control, not the user dragging
			return
        }
        
        // Switch the indicator when more than 50% of the previous/next page is visible
		let pageWidth = self.scrollView.frame.size.width
        let page = Int(floor((self.scrollView.contentOffset.x - pageWidth / 2) / pageWidth) + 1)
        self.pageControl.currentPage = page
        
        // load the visible page and the page on either side of it (to avoid flashes when the user starts scrolling)
		self.loadScrollViewWithPage(page - 1)
        self.loadScrollViewWithPage(page)
        self.loadScrollViewWithPage(page + 1)
        
        // A possible optimization would be to unload the views+controllers which are no longer visible
    }
    
    // At the begin of scroll dragging, reset the boolean used when scrolls originate from the UIPageControl
    func scrollViewWillBeginDragging(scrollView: UIScrollView) {
        self.pageControlUsed = false
    }
    
    // At the end of scroll animation, reset the boolean used when scrolls originate from the UIPageControl
    func scrollViewDidEndDecelerating(scrollView: UIScrollView) {
        self.pageControlUsed = false
    }
    
    @IBAction func changePage(sender: UIPageControl) {
        
        let page = self.pageControl.currentPage
        
        // load the visible page and the page on either side of it (to avoid flashes when the user starts scrolling)

        self.loadScrollViewWithPage(page - 1)
        self.loadScrollViewWithPage(page)
        self.loadScrollViewWithPage(page + 1)
        
        // updage the scroll view to the appropriate page
        var frame = self.scrollView.frame
        frame.origin.x = frame.size.width * CGFloat(page)
        frame.origin.y = 0.0
        self.scrollView.scrollRectToVisible(frame, animated: true)
        
        // Set the boolean used when scrolls originate from the UIPageControl. See scrollViewDidScroll: above.
        pageControlUsed = true

    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }


}

