//
//  DetailViewController.swift
//  SimulatorDemo
//
//  Created by Kevin McNeish on 11/24/14.
//  Copyright (c) 2014 Oak Leaf Enterprises, Inc. All rights reserved.
//

import UIKit

class DetailViewController: UIViewController, UITableViewDelegate, UITableViewDataSource {
    
    @IBOutlet weak var imageView: UIImageView!
    @IBOutlet weak var textView: UITextView!
    @IBOutlet weak var lblTenure: UILabel!
    @IBOutlet weak var tableView: UITableView!
    
    var chiefEntity: ChiefEntity?
    
    // On iPhones, this is used to fill the "Key Products During Tenure" row
    // On iPads, this is used to fill the product list tableview
    var dataSource = Array<String>()
    
    var productFamily = ProductFamily()
    var productList = Array<String>()

    var detailItem: AnyObject? {
        didSet {
            // Update the view.
            self.configureView()
        }
    }

    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.configureView()
    }
    
    func configureView() {
        // Update the user interface for the detail item.
        if let chiefEntity = self.chiefEntity {
            
            self.title = chiefEntity.fullName;
            self.imageView.image = UIImage(named: chiefEntity.picture)
            self.lblTenure.text = chiefEntity.tenure
            self.textView.text = chiefEntity.bio
            self.textView.font = UIFont.systemFontOfSize(15.0)
            self.textView.scrollRangeToVisible(NSMakeRange(0, 0))
            
            if UIDevice.currentDevice().userInterfaceIdiom == .Pad {
                
                self.dataSource = self.productFamily.getProductsForChief(chiefEntity.chiefID)
                self.tableView.reloadData()
            }
            else {
                self.dataSource = ["Key Products During Tenure"]
            }
            
            // Get the list of products for the current chief
            self.productList = self.productFamily.getProductsForChief(chiefEntity.chiefID)
        }

    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        
        let alertController = UIAlertController(title: "Memory Warning", message: "We're getting low on memory!", preferredStyle: .Alert)
        
        let OKAction = UIAlertAction(title: "OK", style: .Default) { (action) in
            // ...
        }
        alertController.addAction(OKAction)
        
        self.presentViewController(alertController, animated: true) {
            // ...
        }
    }
    
    // Detecting a shake
    override func motionEnded(motion: UIEventSubtype, withEvent event: UIEvent) {
        
        let alertController = UIAlertController(title: "Shake Detected", message: "Device received a shake!", preferredStyle: .Alert)
        
        let OKAction = UIAlertAction(title: "OK", style: .Default) { (action) in
            // ...
        }
        alertController.addAction(OKAction)
        
        self.presentViewController(alertController, animated: true) {
            // ...
        }
    }

    // MARK: - Table View
    func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        return 1
    }
    
    func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        return dataSource.count
    }
    
    func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier("ProductsCell", forIndexPath: indexPath) as! UITableViewCell
        
        // Configure the cell...
        
        cell.textLabel?.text = dataSource[indexPath.row]
        
        return cell
    }
    
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        
        if segue.identifier == "SegueToProducts" {
            
            var vc = segue.destinationViewController as! ProductsTableViewController
            vc.productList = self.productList
        }
    }

}

