//
//  ColorThemeCollectionViewController.swift
//  TicTacToe
//
//  Created by Kevin McNeish on 4/17/15.
//  Copyright (c) 2015 Oak Leaf Enterprises, Inc. All rights reserved.
//

import UIKit
import StoreKit
import TicTacToeFramework

class ColorThemeCollectionViewController: UICollectionViewController {
    
    var themeBundleList = Array<ThemeBundleEntity>()
    var theme = Theme()
    var themeDictionary = Dictionary<Int, Array<ThemeEntity>>()
    let reuseIdentifier = "ColorThemeCell"
    @IBOutlet weak var cell: ColorThemeCell!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        // Get unpurchased products from the App Store
        ThemeBundle.sharedInstance.getUnpurchasedProductsFromAppStore()
        
        // Register cell classes
        self.collectionView!.registerClass(UICollectionViewCell.self, forCellWithReuseIdentifier: self.reuseIdentifier)
        
        // Do any additional setup after loading the view.
        self.navigationItem.backBarButtonItem = UIBarButtonItem(title: "", style: UIBarButtonItemStyle.Plain, target: nil, action: nil)
        
        let collectionViewLayout = self.collectionView?.collectionViewLayout as! UICollectionViewFlowLayout
        
        collectionViewLayout.itemSize = CGSizeMake(115, 115)
    }
    
    override func viewWillAppear(animated: Bool) {
        super.viewWillAppear(animated)
        
        // Get purchased theme bundles
        self.themeBundleList = ThemeBundle.sharedInstance.getPurchasedThemeBundles()
        
        // Create a dictionary of all theme bundle IDs and their
        // associated themEntities
        for (index, themeBundleEntity) in enumerate(self.themeBundleList) {
            var themeList = self.theme.getThemesByThemeBundleID(themeBundleEntity.themeBundleID)
            self.themeDictionary[index] = themeList
        }
        self.collectionView?.reloadData()
    }
    
    override func collectionView(collectionView: UICollectionView, cellForItemAtIndexPath indexPath: NSIndexPath) -> UICollectionViewCell {
        
        let cell = collectionView.dequeueReusableCellWithReuseIdentifier("Cell", forIndexPath: indexPath) as! UICollectionViewCell
        
        let themeEntity = self.themeDictionary[indexPath.section]![indexPath.row]
        
        // Configure the cell
        if let draw2D = cell.viewWithTag(102) as? Draw2DPreview {
            draw2D.themeEntity = themeEntity
            draw2D.setNeedsDisplay()
            draw2D.setNeedsLayout()
        }
        
        if let label = cell.viewWithTag(101) as? UILabel {
            
            label.text = themeEntity.name
        }
        
        return cell
    }
    
    override func collectionView(collectionView: UICollectionView, viewForSupplementaryElementOfKind kind: String, atIndexPath indexPath: NSIndexPath) -> UICollectionReusableView {
        
        var reusableView: UICollectionReusableView!
        
        if kind == UICollectionElementKindSectionHeader {
            
            let headerView = collectionView.dequeueReusableSupplementaryViewOfKind(UICollectionElementKindSectionHeader, withReuseIdentifier: "HeaderView", forIndexPath: indexPath) as! ThemeCollectionHeaderView
            
            let title = themeBundleList[indexPath.section].themeBundleName
            
            headerView.lblThemeTitle.text = title
            reusableView = headerView
        }
        else if kind == UICollectionElementKindSectionFooter  {
            
            let footerView = collectionView.dequeueReusableSupplementaryViewOfKind(UICollectionElementKindSectionFooter, withReuseIdentifier: "FooterView", forIndexPath: indexPath) as! ThemeCollectionFooterView
            
            //            if !InAppPurchaseManager.sharedInstance.canMakePurchases {
            //                footerView.btnMoreColorThemes.hidden = true
            //            }
            
            reusableView = footerView
        }
        
        return reusableView
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    // MARK: UICollectionViewDataSource
    override func numberOfSectionsInCollectionView(collectionView: UICollectionView) -> Int {
        return themeBundleList.count
    }
    
    override func collectionView(collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        
        return themeDictionary[section]!.count
    }
    
    // MARK: UICollectionViewDelegate
    
    override func collectionView(collectionView: UICollectionView, didSelectItemAtIndexPath indexPath: NSIndexPath) {
        
        let themeID = self.themeDictionary[indexPath.section]![indexPath.row].themeID
        
        globalThemeEntity = globalTheme.getThemeByID(themeID.integerValue)
        
        self.performSegueWithIdentifier("UnwindToGameSetupSegue", sender: nil)
    }
    
    @IBAction func moreColorThemes(sender: AnyObject) {
        
        self.performSegueWithIdentifier("ThemeStoreSegue", sender: self)
    }
    
    // Uncomment this method to specify if the specified item should be highlighted during tracking
    override func collectionView(collectionView: UICollectionView, shouldHighlightItemAtIndexPath indexPath: NSIndexPath) -> Bool {
        return true
    }
    
}
