//
//  ColorThemeStoreCollectionViewController.swift
//  TicTacToe
//
//  Created by Kevin McNeish on 4/30/15.
//  Copyright (c) 2015 Oak Leaf Enterprises, Inc. All rights reserved.
//

import UIKit
import TicTacToeFramework
import StoreKit

class ColorThemeStoreCollectionViewController: UICollectionViewController {
    
    //var themeBundleEntity: ThemeBundleEntity!
    var themeBundle = ThemeBundle()
    var selectedEntity: ThemeBundleEntity!
    var theme = Theme()
    var themeList = Array<ThemeEntity>()
    let reuseIdentifier = "ColorThemeCell"
    var priceString = ""
    let buyNowString = NSLocalizedString("Buy now", comment: "")
    var headerButton: UIButton!
    var footerButton: UIButton!
    var notificationToken: NSObjectProtocol!
    
    @IBOutlet weak var cell: ColorThemeCell!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.themeList = self.theme.getThemesByThemeBundleID(self.selectedEntity.product!.productIdentifier)
        
        // Register cell classes
        self.collectionView!.registerClass(UICollectionViewCell.self, forCellWithReuseIdentifier: reuseIdentifier)
        
        let collectionViewLayout = self.collectionView?.collectionViewLayout as! UICollectionViewFlowLayout
        
        collectionViewLayout.itemSize = CGSizeMake(115, 115)
        
        // Get localized product information
        self.title = self.selectedEntity.product!.localizedTitle
        
        self.priceString = InAppPurchaseManager.sharedInstance.getLocalizedPriceStringForProduct(self.selectedEntity.product!)
        
        appDelegate.window?.bringSubviewToFront(appDelegate.activityIndicator)
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    override func viewWillDisappear(animated: Bool) {
        appDelegate.activityIndicator.stopAnimating()
        self.setBuyNowButtonEnabled(true)
    }
    
    deinit {
        if self.notificationToken != nil {
            NSNotificationCenter.defaultCenter().removeObserver(self.notificationToken)
        }
    }
    
    @IBAction func buyNow(sender: AnyObject) {
        
        if InAppPurchaseManager.sharedInstance.canMakePurchases {
            
            appDelegate.activityIndicator.startAnimating()
            self.setBuyNowButtonEnabled(false)
            
            // Register for notification of In-App Purchase.
            self.notificationToken =
                NSNotificationCenter.defaultCenter().addObserverForName(THEMEBUNDLE_PURCHASED, object: nil, queue: NSOperationQueue.mainQueue()) { [unowned self]notification in
                    
                    NSNotificationCenter.defaultCenter().removeObserver(self.notificationToken)
                    
                    self.setBuyNowButtonEnabled(true)
                    appDelegate.activityIndicator.stopAnimating()
                    
                    let success = notification.userInfo!["success"] as! Bool
                    
                    if success {
                        // Purchase succeeded!
                        self.navigationController?.popToRootViewControllerAnimated(true)
                    }
                    else {
                        
                        // Purchase failed!
                        // Display an alert only if there is an error message
                        // (Some errors such as user cancel are ignored)
                        if let errorMessage = notification.userInfo!["errorMessage"] as! String? {
                            
                            let alertController = UIAlertController(title: NSLocalizedString("Unable to Complete Purchase", comment: ""), message: errorMessage, preferredStyle: .Alert)
                            
                            let OKAction = UIAlertAction(title: "OK", style: .Default) { (action) in
                                // ...
                            }
                            
                            alertController.addAction(OKAction)
                            
                            self.presentViewController(alertController, animated: true) {
                                // ...
                            }
                        }
                    }
            }
            // Purchase the product
            ThemeBundle.sharedInstance.purchaseProduct(self.selectedEntity.product!)
        }
        else {
            
            let alertController = UIAlertController(title: NSLocalizedString("Unauthorized Purchase", comment: ""), message: NSLocalizedString("You are not authorized to make purchases on this device.", comment: ""), preferredStyle: .Alert)
            
            let OKAction = UIAlertAction(title: "OK", style: .Default) { (action) in
                // ...
            }
            
            alertController.addAction(OKAction)
            
            self.presentViewController(alertController, animated: true) {
                // ...
            }
        }
    }
    
    // Make purchased product available
    func provideContentForTransaction(transaction: SKPaymentTransaction) {
        
        let productID = transaction.payment.productIdentifier
        
        if productID.rangeOfString("Animation") != nil {
            // An animation was purchased
        }
        else {
            // A color theme was purchased
            self.themeBundle.markBundleAsPurchased(productID)
        }
    }
    
    // Enable/disable the Buy Now buttons
    func setBuyNowButtonEnabled(enabled: Bool) {
        
        if self.headerButton != nil {
            self.headerButton.enabled = enabled
        }
        if self.footerButton != nil {
            self.footerButton.enabled = enabled
        }
    }
    
    // MARK: UICollectionViewDataSource
    
    override func numberOfSectionsInCollectionView(collectionView: UICollectionView) -> Int {
        return 1
    }
    
    override func collectionView(collectionView: UICollectionView, numberOfItemsInSection section: Int) -> Int {
        return self.themeList.count
    }
    
    override func collectionView(collectionView: UICollectionView, cellForItemAtIndexPath indexPath: NSIndexPath) -> UICollectionViewCell {
        
        
        let cell = collectionView.dequeueReusableCellWithReuseIdentifier("Cell", forIndexPath: indexPath) as! UICollectionViewCell
        
        let themeEntity = self.themeList[indexPath.row]
        
        // Configure the cell
        if let draw2D = cell.viewWithTag(102) as? Draw2DPreview {
            draw2D.themeEntity = themeEntity
            draw2D.setNeedsDisplay()
            draw2D.setNeedsLayout()
        }
        
        if let label = cell.viewWithTag(101) as? UILabel {
            
            label.text = themeEntity.name
        }
        
        return cell
    }
    
    override func collectionView(collectionView: UICollectionView, viewForSupplementaryElementOfKind kind: String, atIndexPath indexPath: NSIndexPath) -> UICollectionReusableView {
        
        var reusableView: UICollectionReusableView!
        
        
        let buyString = String(format: "\(self.buyNowString): %@", self.priceString)
        
        if kind == UICollectionElementKindSectionHeader {
            
            let headerView = collectionView.dequeueReusableSupplementaryViewOfKind(UICollectionElementKindSectionHeader, withReuseIdentifier: "HeaderView", forIndexPath: indexPath) as! ThemeStoreCollectionHeaderFooterView
            
            headerView.btnBuyNow.setTitle(buyString, forState: UIControlState.Normal)
            
            reusableView = headerView
            self.headerButton = headerView.btnBuyNow
        }
        else if kind == UICollectionElementKindSectionFooter  {
            
            let footerView = collectionView.dequeueReusableSupplementaryViewOfKind(UICollectionElementKindSectionFooter, withReuseIdentifier: "FooterView", forIndexPath: indexPath) as! ThemeStoreCollectionHeaderFooterView
            
            footerView.btnBuyNow.setTitle(buyString, forState: UIControlState.Normal)
            
            reusableView = footerView
            self.footerButton = footerView.btnBuyNow
        }
        
        return reusableView
    }
    
}
