//
//  ColorThemeStoreTableViewController.swift
//  TicTacToe
//
//  Created by Kevin McNeish on 4/29/15.
//  Copyright (c) 2015 Oak Leaf Enterprises, Inc. All rights reserved.
//

import UIKit
import TicTacToeFramework
import StoreKit

class ColorThemeStoreTableViewController: UITableViewController {
    
    var themeBundleEntity: ThemeBundleEntity!
    var validProducts = Array<SKProduct>()
    var selectedEntity: ThemeBundleEntity!
    var notificationToken: NSObjectProtocol!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        if ThemeBundle.sharedInstance.unpurchasedBundleList == nil {
            
            // Register for notification of In-App Purchase products retrieved.
            // Retrieval is initiated on the previos view controller
            // Need to register for notification in case taking a long time to retrieve products
            self.notificationToken =
                NSNotificationCenter.defaultCenter().addObserverForName(THEMEBUNDLES_RETRIEVED, object: nil, queue: NSOperationQueue.mainQueue()) { [unowned self]notification in
                    
                    NSNotificationCenter.defaultCenter().removeObserver(self.notificationToken)
                    appDelegate.activityIndicator.stopAnimating()
                    self.tableView.reloadData()
            }
            appDelegate.window?.bringSubviewToFront(appDelegate.activityIndicator)
            appDelegate.activityIndicator.startAnimating()
        }
        
        self.navigationItem.backBarButtonItem = UIBarButtonItem(title: "", style: UIBarButtonItemStyle.Plain, target: nil, action: nil)
    }
    
    deinit {
        if self.notificationToken != nil {
            NSNotificationCenter.defaultCenter().removeObserver(self.notificationToken)
        }
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    // MARK: - Table view data source
    
    override func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        // Return the number of sections.
        
        return 1
    }
    
    override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        // Return the number of rows in the section.
        return ThemeBundle.sharedInstance.unpurchasedBundleList?.count ?? 0
    }
    
    override func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCellWithIdentifier("ThemeBundleCell", forIndexPath: indexPath) as! UITableViewCell
        
        // Configure the cell...
        let themeBundleEntity = ThemeBundle.sharedInstance.unpurchasedBundleList![indexPath.row]
        
        cell.textLabel?.text = themeBundleEntity.product!.localizedTitle
        
        cell.textLabel?.textColor = UIColor(red: CGFloat(themeBundleEntity.themeRed)/255, green: CGFloat(themeBundleEntity.themeGreen)/255, blue: CGFloat(themeBundleEntity.themeBlue)/255, alpha: 1.0)
        
        return cell
    }
    
    override func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
        
        self.selectedEntity = ThemeBundle.sharedInstance.unpurchasedBundleList![indexPath.row]
        self.performSegueWithIdentifier("PurchaseThemeBundleSegue", sender: nil)
    }
    
    // MARK: - Navigation
    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        
        if segue.identifier == "PurchaseThemeBundleSegue" {
            
            let vc = segue.destinationViewController as! ColorThemeStoreCollectionViewController
            
            vc.selectedEntity = self.selectedEntity
        }
    }
    
}
