//
//  GameSetupTableViewController.swift
//  TicTacToe
//
//  Created by Kevin McNeish on 3/23/15.
//  Copyright (c) 2015 Oak Leaf Enterprises, Inc. All rights reserved.
//

import UIKit
import TicTacToeFramework
import StoreKit

protocol ThemeChangeDelegate {
    func setGlobalTheme()
}

enum GameSetupCell: String {
    
    case ColorThemeCell = "ColorThemeCell"
    case StartCell = "StartCell"
    case RestorePurchasesCell = "RestorePurchasesCell"
}

class GameSetupTableViewController: UITableViewController {
    
    var txtPlayer1Name: UITextField!
    var txtPlayer2Name: UITextField?
    var delegate: ThemeChangeDelegate!
    @IBOutlet weak var lblThemeName: UILabel!
    var selectedTextColor: UIColor!
    var notificationToken: NSObjectProtocol?
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
         self.navigationItem.backBarButtonItem = UIBarButtonItem(title: "", style: UIBarButtonItemStyle.Plain, target: nil, action: nil)
        
        self.setColorTheme()
        
        appDelegate.window?.bringSubviewToFront(appDelegate.activityIndicator)
    }
    
    override func viewWillLayoutSubviews() {
        self.centerTableView()
    }
    
    override func viewWillDisappear(animated: Bool) {
        appDelegate.activityIndicator.stopAnimating()
    }
    
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    @IBAction func hideKeyboard(sender: AnyObject) {
        self.view.endEditing(true)
    }

    // MARK: - Table view data source
    override func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        // Return the number of sections.
        return 1
    }
    
    override func tableView(tableView: UITableView, heightForHeaderInSection section: Int) -> CGFloat {
        return 8.0
    }

    override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        
        return 3
    }

    override func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        
        var cell:UITableViewCell!
        var attributes = [NSFontAttributeName:UIFont.systemFontOfSize(17.0)]

        // Configure the cell...
        switch indexPath.row {
            
        case 0:
            cell = tableView.dequeueReusableCellWithIdentifier(GameSetupCell.ColorThemeCell.rawValue, forIndexPath: indexPath) as! UITableViewCell
                
            cell.textLabel?.textColor = globalThemeEntity.tintX
            cell.textLabel?.text = globalThemeEntity.name + " " + NSLocalizedString("Theme", comment: "Color theme")
        case 1:
            cell = tableView.dequeueReusableCellWithIdentifier(GameSetupCell.RestorePurchasesCell.rawValue, forIndexPath: indexPath) as! UITableViewCell
            
        case 2:
            cell = tableView.dequeueReusableCellWithIdentifier(GameSetupCell.StartCell.rawValue, forIndexPath: indexPath) as! UITableViewCell
        default:
            break
        }

        return cell
    }
    
//    override func tableView(tableView: UITableView, heightForRowAtIndexPath indexPath: NSIndexPath) -> CGFloat {
//
//        return 46
//    }
    
    func restorePurchases() {
        
        let alertController = UIAlertController(title: NSLocalizedString("Restore Purchases", comment: ""), message: nil, preferredStyle: .ActionSheet)
        
        let CancelAction = UIAlertAction(title: "Cancel", style: .Cancel) { (action) in
            // ...
        }
        alertController.addAction(CancelAction)
        
        let OKAction = UIAlertAction(title: "Restore", style: .Default) { (action) in
            
            // Disable the Restore Purchases cell so they don't tap it multiple times
            self.setRestorePurchaseCellEnabled(false)
            
            // Register the view controller for notification of completion of all restored purchases
            self.notificationToken =
                NSNotificationCenter.defaultCenter().addObserverForName(THEMEBUNDLE_ALL_PURCHASES_RESTORED, object: nil, queue: NSOperationQueue.mainQueue()) { [unowned self] notification in
                    
                    // Unregister the observer
                    NSNotificationCenter.defaultCenter().removeObserver(self.notificationToken!)
                    
                    appDelegate.activityIndicator.stopAnimating()
                    // Enable the Restore Purchases
                    self.setRestorePurchaseCellEnabled(true)
                    
                    let userInfo = notification.userInfo!
                    let success = userInfo["success"] as! Bool
                    if success {
                        
                        var title = ""
                        var message = ""
                        
                        let count = userInfo["count"] as! Int
                        
                        if count > 0 {
                            title = NSLocalizedString("Restore Successful", comment: "")
                            message = NSLocalizedString("Your purchases have been restored.", comment:"")
                        }
                        else {
                            title = NSLocalizedString("Restore Purchases", comment: "")
                            message = NSLocalizedString("There were no purchases to restore.", comment:"")
                        }
                        
                        // Reload data to display the newly restored products
                        let alertController = UIAlertController(title: title, message: message, preferredStyle: .Alert)
                        
                        let OKAction = UIAlertAction(title: "OK", style: .Default) { (action) in
                            
                            if count > 0 {
                                self.performSegueWithIdentifier("SelectColorThemeSegue", sender: self)
                            }
                        }
                        
                        alertController.addAction(OKAction)
                        
                        self.presentViewController(alertController, animated: true) {
                            // ...
                        }
                    }
                    else {
                        // Display an alert only if there is an error message
                        // (Some errors such as user cancel are ignored)
                        if let errorMessage = userInfo["errorMessage"] as! String? {
                            
                            let alertController = UIAlertController(title: NSLocalizedString("Unable to Complete Restore", comment: ""), message: errorMessage, preferredStyle: .Alert)
                            
                            let OKAction = UIAlertAction(title: "OK", style: .Default) { (action) in
                                // ...
                            }
                            
                            alertController.addAction(OKAction)
                            
                            self.presentViewController(alertController, animated: true) {
                                // ...
                            }
                        }
                    }
            }
            
            appDelegate.activityIndicator.startAnimating()
            
            // Restore all in-app purchases
            ThemeBundle.sharedInstance.restoreAllInAppPurchases()
        }
        alertController.addAction(OKAction)
        
        self.presentViewController(alertController, animated: true) {
            // ...
        }
    }
    
    func setRestorePurchaseCellEnabled(enabled: Bool) {
        
        // Disable the Restore Purchases cell so they don't tap it multiple times
        let restorePurchasesCell = self.tableView.cellForRowAtIndexPath(NSIndexPath(forRow: 1, inSection: 0))
        
        restorePurchasesCell?.userInteractionEnabled = enabled
        restorePurchasesCell?.textLabel?.enabled = enabled

    }
    
    @IBAction func cancelColorTheme(segue:UIStoryboardSegue) {
        
    }
    
    @IBAction func saveColorTheme(segue:UIStoryboardSegue) {
        
        // Get a reference to the Color Theme view controller
        let vc = segue.sourceViewController as! ColorThemeCollectionViewController
        
        // Get the selected themeID and save it in settings
        Settings.sharedInstance.themeID = globalThemeEntity.themeID.integerValue
        Settings.sharedInstance.saveEntities()
        
        // Set the selected color theme
        self.setColorTheme()
        
        // Reload the table view
        self.tableView.reloadData()
        
        // Set the global theme on the tic-tac-toe view controller
        self.delegate.setGlobalTheme()
    }
    
    func setColorTheme() {
        
        // Set the selected text color
        self.selectedTextColor = globalThemeEntity.lightContentSetting ? UIColor.blackColor() : UIColor.whiteColor()
		UISegmentedControl.appearance().setTitleTextAttributes([NSForegroundColorAttributeName: self.selectedTextColor], forState: UIControlState.Selected)
    }
    
    override func tableView(tableView: UITableView, didSelectRowAtIndexPath indexPath: NSIndexPath) {
        
        if indexPath.row == 1 {
            self.restorePurchases()
        }
        else if indexPath.row == 2 {
            self.performSegueWithIdentifier("SaveSettingsNewGameSegue", sender: nil)
        }
    }
    
    deinit {
        NSNotificationCenter.defaultCenter().removeObserver(self)
    }

}
