//
//  ViewController.swift
//  TicTacToe
//
//  Created by Kevin McNeish on 3/15/15.
//  Copyright (c) 2015 Oak Leaf Enterprises, Inc. All rights reserved.
//

import UIKit
import Social
import TicTacToeFramework
	
	let lightContent = true

class TicTacToeViewController: UIViewController, TicTacToeDelegate, ThemeChangeDelegate {

    var ticTacToe = TicTacToe()
    var theme = Theme()
    var gameOver = false
    
    @IBOutlet weak var lblPlayer: UILabel!
    
    @IBOutlet weak var lblA: UILabel!
    @IBOutlet weak var lblB: UILabel!
    @IBOutlet weak var lblC: UILabel!
    @IBOutlet weak var lblD: UILabel!
    @IBOutlet weak var lblE: UILabel!
    @IBOutlet weak var lblF: UILabel!
    @IBOutlet weak var lblG: UILabel!
    @IBOutlet weak var lblH: UILabel!
    @IBOutlet weak var lblI: UILabel!
    
    @IBOutlet weak var resultsView: UIView!
    @IBOutlet weak var optionsView: UIView!
    @IBOutlet weak var lblWinnerName: UILabel!
    @IBOutlet weak var lblWinnerText: UILabel!
    @IBOutlet weak var lblPlayer1Stats: UILabel!
    @IBOutlet weak var lblPlayer2Stats: UILabel!
    @IBOutlet weak var lblTiesStats: UILabel!
    @IBOutlet weak var btnPlayAgain: UIButton!
    @IBOutlet weak var btnTwitter: UIButton!
    @IBOutlet weak var btnFacebook: UIButton!
    @IBOutlet weak var btnGameCenter: UIButton!
    @IBOutlet weak var btnSettings: UIButton!
    @IBOutlet weak var draw2D: Draw2D!
    
    var colorThemeStoreTableViewController: ColorThemeStoreTableViewController!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        self.ticTacToe.delegate = self
        
         self.navigationItem.backBarButtonItem = UIBarButtonItem(title: "", style: UIBarButtonItemStyle.Plain, target: nil, action: nil)
        
        // Set the global theme entity
        globalThemeEntity = globalTheme.getThemeByID(Settings.sharedInstance.themeID)
        self.setGlobalTheme()
    }
    
    override func viewWillAppear(animated: Bool) {
    	super.viewWillAppear(animated)

        self.newGame()
    }
    
    // Start a new game
    @IBAction func newGame() {
        	
        self.clearLabels()
        self.gameOver = false
        self.ticTacToe.setupNewGame()
        self.showCurrentPlayer()
    }
    
    // This method gets called directly as well as via TicTacToeDelegate
    func setGlobalTheme() {
        
        self.draw2D.setNeedsDisplay()
        
        // This code is necessary if the user changes the tint in
        // the middle of a game and then clicks X to cancel
        // other Settings changes
        
        // Show the current player in the global tint
        self.showCurrentPlayer()
        
        // set the tint of the "X" labels
        if self.lblA.text == "X" {
        	self.lblA.textColor = globalThemeEntity.tintX
        }
        if self.lblB.text == "X" {
        	self.lblB.textColor = globalThemeEntity.tintX
        }
        if self.lblC.text == "X" {
        	self.lblC.textColor = globalThemeEntity.tintX
        }
        if self.lblD.text == "X" {
        	self.lblD.textColor = globalThemeEntity.tintX
        }
        if self.lblE.text == "X" {
        	self.lblE.textColor = globalThemeEntity.tintX
        }
        if self.lblF.text == "X" {
        	self.lblF.textColor = globalThemeEntity.tintX
        }
        if self.lblG.text == "X" {
        	self.lblG.textColor = globalThemeEntity.tintX
        }
        if self.lblH.text == "X" {
        	self.lblH.textColor = globalThemeEntity.tintX
        }
        if self.lblI.text == "X" {
        	self.lblI.textColor = globalThemeEntity.tintX
        }
        
        // set the tint of the "O" labels
        if self.lblA.text == "O" {
            self.lblA.textColor = globalThemeEntity.tintO
        }
        if self.lblB.text == "O" {
            self.lblB.textColor = globalThemeEntity.tintO
        }
        if self.lblC.text == "O" {
            self.lblC.textColor = globalThemeEntity.tintO
        }
        if self.lblD.text == "O" {
            self.lblD.textColor = globalThemeEntity.tintO
        }
        if self.lblE.text == "O" {
            self.lblE.textColor = globalThemeEntity.tintO
        }
        if self.lblF.text == "O" {
            self.lblF.textColor = globalThemeEntity.tintO
        }
        if self.lblG.text == "O" {
            self.lblG.textColor = globalThemeEntity.tintO
        }
        if self.lblH.text == "O" {
            self.lblH.textColor = globalThemeEntity.tintO
        }
        if self.lblI.text == "O" {
            self.lblI.textColor = globalThemeEntity.tintO
        }
        self.btnPlayAgain.tintColor = globalThemeEntity.tintX
        self.btnTwitter.tintColor = globalThemeEntity.tintX
        self.btnFacebook.tintColor = globalThemeEntity.tintX
        self.btnGameCenter.tintColor = globalThemeEntity.tintX
        self.btnSettings.tintColor = globalThemeEntity.tintX
    }
    
    @IBAction func hideResultsNewGame(sender: UIButton) {
        self.hideResultsView()
        self.newGame()
    }
    
    @IBAction func showSettings(sender: AnyObject) {
        self.performSegueWithIdentifier("SettingsSegue", sender: self)
    }
    
    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        if segue.identifier == "SettingsSegue" {
            let nc = segue.destinationViewController as! UINavigationController
            let vc = nc.viewControllers[0] as! GameSetupTableViewController
            vc.delegate = self
        }
    }
    
    @IBAction func hideResultsShowSettings(sender: UIButton) {
        
        self.hideResultsView()
        self.showSettings(sender)
    }
    
    @IBAction func hideResultsShowGameCenter(sender: UIButton) {
    }
    
    @IBAction func hideResultsShowFacebook(sender: UIButton) {
        
        let message = self.buildSocialMessage()

        // check if there is an account for Facebook
        if SLComposeViewController.isAvailableForServiceType(SLServiceTypeFacebook)
        {
            let fbController = SLComposeViewController(forServiceType: SLServiceTypeFacebook)
            fbController.setInitialText(message)
            
            fbController.addImage(UIImage(named: "icon.png"))
            self.presentViewController(fbController, animated: true, completion: nil)
        }
    }
    
    @IBAction func hideResultsShowTwitter(sender: UIButton) {
    
        let message = self.buildSocialMessage()
        
        // check if there is an account for Twitter
        if SLComposeViewController.isAvailableForServiceType(SLServiceTypeTwitter)
        {
            let twitterController = SLComposeViewController(forServiceType: SLServiceTypeTwitter)
            twitterController.setInitialText(message)
            
            twitterController.addImage(UIImage(named: "icon.png"))
            self.presentViewController(twitterController, animated: true, completion: nil)
        }
    }
    
    func buildSocialMessage() -> String {
        
        if count(self.lblWinnerText.text!) > 0 {
            return "I have a \(self.lblWinnerText.text!) in Tic-Tac-Toe: "
        }
        else {
            return "I'm playing Tic-Tac-Toe: "
        }
    }
    
    func playerMoved(player: String, position: Position, win: Array<Position>?, gameOver: Bool) {
        self.setPositionLabel(position, value: player)
        self.checkGameOver((player, position, win, gameOver))
    }
    
    func setPositionLabel(position: Position, value: String) {
        if position == .a {
            self.setLabel(self.lblA, mark: value)
        }
        else if position == .b {
            self.setLabel(self.lblB, mark: value)
        }
        else if position == .c {
            self.setLabel(self.lblC, mark: value)
        }
        else if position == .d {
            self.setLabel(self.lblD, mark: value)
        }
        else if position == .e {
            self.setLabel(self.lblE, mark: value)
        }
        else if position == .f {
            self.setLabel(self.lblF, mark: value)
        }
        else if position == .g {
            self.setLabel(self.lblG, mark: value)
        }
        else if position == .h {
            self.setLabel(self.lblH, mark: value)
        }
        else if position == .i {
            self.setLabel(self.lblI, mark: value)
        }
    }
    
    func setLabel(label: UILabel, mark: String) {
        if mark == "X" {
            label.textColor = globalThemeEntity.tintX
        }
        else {
            label.textColor = globalThemeEntity.tintO
        }
        label.text = mark
    }
    
    func clearLabels() {
        self.lblA.text = ""
        self.lblB.text = ""
        self.lblC.text = ""
        self.lblD.text = ""
        self.lblE.text = ""
        self.lblF.text = ""
        self.lblG.text = ""
        self.lblH.text = ""
        self.lblI.text = ""
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    // Select the specified position for the current player
    func selectPositionForCurrentPlayer(position: Position) {
        
        // Make sure the position is open
        if !self.gameOver && self.ticTacToe.isPositionOpen(position) &&
        	(self.ticTacToe.isCurrentPlayerHuman) {
                
            // Make a move for the current user in the specified position
            let result = self.ticTacToe.currentPlayerMove(position)
            
            self.setPositionLabel(result.position, value: result.player)
            self.checkGameOver(result)

            if !result.gameOver && result.win == nil {
                self.showCurrentPlayer()
            }
        }
    }
    
    // Returns true if the game is over, otherwise returns false
    func checkGameOver(result: (player: String, position: Position, win: Array<Position>?, gameOver: Bool)) {

        if result.win != nil {
            
            self.gameOver = true
            
            // Animate the win
            UIView.animateWithDuration(0.5, delay: 0.0, options: UIViewAnimationOptions.Repeat, animations: { () -> Void in
                UIView.setAnimationRepeatCount(3.0)
                
                for position in result.win! {
                    switch position {
                    case .a:
                        self.lblA.alpha = 0
                    case .b:
                        self.lblB.alpha = 0
                    case .c:
                        self.lblC.alpha = 0
                    case .d:
                        self.lblD.alpha = 0
                    case .e:
                        self.lblE.alpha = 0
                    case .f:
                        self.lblF.alpha = 0
                    case .g:
                        self.lblG.alpha = 0
                    case .h:
                        self.lblH.alpha = 0
                    case .i:
                        self.lblI.alpha = 0
                    }
                }
                
            }, completion: { (animationFinished) -> Void in
                for position in result.win! {
                    switch position {
                    case .a:
                        self.lblA.alpha = 1
                    case .b:
                        self.lblB.alpha = 1
                    case .c:
                        self.lblC.alpha = 1
                    case .d:
                        self.lblD.alpha = 1
                    case .e:
                        self.lblE.alpha = 1
                    case .f:
                        self.lblF.alpha = 1
                    case .g:
                        self.lblG.alpha = 1
                    case .h:
                        self.lblH.alpha = 1
                    case .i:
                        self.lblI.alpha = 1
                    }
                }
                self.displayResultsView()
            })
        }
        else if result.gameOver{
            self.gameOver = true
            self.displayResultsView()
        }
        else  {
            self.showCurrentPlayer()
        }
    }
    
    func displayResultsView() {
        
        self.view.bringSubviewToFront(self.resultsView)
        
    	UIView.animateWithDuration(0.75, animations: { () -> Void in
        
        self.resultsView.alpha = 1

        self.navigationController?.navigationBarHidden = true
        
        self.lblWinnerName.text = self.ticTacToe.winnerName
        self.lblWinnerText.text = self.ticTacToe.winnerMessage
        self.lblPlayer1Stats.text = "\(Settings.sharedInstance.player1Name): \(self.ticTacToe.player1Wins)"
        self.lblPlayer2Stats.text = "\(Settings.sharedInstance.player2Name): \(self.ticTacToe.player2Wins)"
        self.lblTiesStats.text = "Ties: \(self.ticTacToe.ties)"
        }) { (Bool) -> Void in
            
            // Animate winning streak, if any
            if count(self.lblWinnerText.text!) > 0 {
                // Animate the win
                UIView.animateWithDuration(0.5, delay: 0.0, options: UIViewAnimationOptions.Repeat, animations: { () -> Void in
                    UIView.setAnimationRepeatCount(3.0)
                    
                    self.lblWinnerText.alpha = 0
                    
                    }, completion: { (animationFinished) -> Void in
                    
                        self.lblWinnerText.alpha = 1
                })
            }
            self.showCurrentPlayer()
        }
    }
    
    func hideResultsView() {
        UIView.animateWithDuration(0.75, animations: { () -> Void in
            
            self.navigationController?.navigationBarHidden = false

            //self.navigationController?.navigationBar.alpha = 1
            self.resultsView.alpha = 0
            
            }) { (Bool) -> Void in
                self.view.sendSubviewToBack(self.resultsView)

        }
    }
    
    func showCurrentPlayer() {
        
        if self.ticTacToe.currentPlayerMark == "X" {
            if Settings.sharedInstance.player1Mark == "X" {
                self.showPlayer1Current()
            }
            else {
                self.showPlayer2Current()
            }
        }
        else {
            if Settings.sharedInstance.player1Mark == "O" {
                self.showPlayer1Current()
            }
            else {
                self.showPlayer2Current()
            }
        }
    }
    
    private func showPlayer1Current() {

        // Display player 1
        if Settings.sharedInstance.player1Mark == "X" {
            self.lblPlayer.textColor = globalThemeEntity.tintX
        }
        else {
            self.lblPlayer.textColor = globalThemeEntity.tintO
        }
        self.lblPlayer.text = "\(Settings.sharedInstance.player1Name): \(Settings.sharedInstance.player1Mark)"
    }
    
    private func showPlayer2Current() {

        // Display player 2
        if Settings.sharedInstance.player2Mark == "X" {
            self.lblPlayer.textColor = globalThemeEntity.tintX
        }
        else {
            self.lblPlayer.textColor = globalThemeEntity.tintO
        }
        self.lblPlayer.text = "\(Settings.sharedInstance.player2Name): \(Settings.sharedInstance.player2Mark)"
    }
    
    override func prefersStatusBarHidden() -> Bool {
        // Hide the status bar if the navigation bar is hidden
        return true
    }

    @IBAction func tapA(sender: AnyObject) {
		self.selectPositionForCurrentPlayer(.a)
    }
    
    @IBAction func tapB(sender: AnyObject) {
        self.selectPositionForCurrentPlayer(.b)
    }

    @IBAction func tapC(sender: AnyObject) {
        self.selectPositionForCurrentPlayer(.c)
    }
    
    @IBAction func tapD(sender: AnyObject) {
       self.selectPositionForCurrentPlayer(.d)
    }
    
    @IBAction func tapE(sender: AnyObject) {
        self.selectPositionForCurrentPlayer(.e)
    }
    
    @IBAction func tapF(sender: AnyObject) {
       self.selectPositionForCurrentPlayer(.f)
    }
    
    @IBAction func tapG(sender: AnyObject) {
       self.selectPositionForCurrentPlayer(.g)
    }
    
    @IBAction func tapH(sender: AnyObject) {
        self.selectPositionForCurrentPlayer(.h)
    }
    
    @IBAction func tapI(sender: AnyObject) {
        self.selectPositionForCurrentPlayer(.i)
    }
    
    @IBAction func saveSettings(segue:UIStoryboardSegue) {
        // This code executes if the user selects New Game or clicks X
        // on the Settings scene

        Settings.sharedInstance.saveEntities()
        self.showCurrentPlayer()
        
        // If returning from Settings and the game is over, start a new game
        if self.gameOver {
            self.newGame()
        }
        
        dispatch_async(dispatch_get_main_queue()) {
            self.dismissViewControllerAnimated(true, completion: nil)
        }
    }
    
    @IBAction func saveSettingsNewGame(segue:UIStoryboardSegue) {
        
        self.saveSettings(segue)
        self.newGame()
    }
}

