//
//  Settings.swift
//  TicTacToe
//
//  Created by Kevin McNeish on 3/24/15.
//  Copyright (c) 2015 Oak Leaf Enterprises, Inc. All rights reserved.
//

import Foundation

private let singletonInstance = Settings()

public class Settings<T: SettingsEntity>: ABusinessObject<T> {
    
    var settingsEntity: SettingsEntity!
    
    public class var sharedInstance: Settings<SettingsEntity> {
        return singletonInstance
    }
    
    public var gameMode: GameMode {
        
        get {
            return settingsEntity.gameModeSetting
        }
        set {
            settingsEntity.gameModeSetting = newValue
        }
    }
    public var player1Name: String {
        get {
            return self.settingsEntity.player1NameSetting
        }
        set {
            self.settingsEntity.player1NameSetting = newValue
        }
    }
    public var player2Name: String {
        get {
            return self.settingsEntity.player2NameSetting
        }
        set {
            self.settingsEntity.player2NameSetting = newValue
        }
    }
    
    public var player1Mark: String {
        get {
            return self.settingsEntity.player1Mark!
        }
        set {
            self.settingsEntity.player1Mark = newValue
        }
    }
    
    public var player2Mark: String {
        get {
            return self.settingsEntity.player2Mark!
        }
        set {
            self.settingsEntity.player2Mark = newValue
        }
    }
    
    public var firstMove: FirstMove {
        
        get {
            return self.settingsEntity.firstMoveSetting
        }
        set {
            self.settingsEntity.firstMoveSetting = newValue
        }
    }
    
    public var difficulty: Difficulty {
        get {
            return self.settingsEntity.difficultySetting
        }
        set {
            self.settingsEntity.difficultySetting = newValue
        }
    }
    
    public var themeID: Int {
        get {
            return self.settingsEntity.themeIDSetting
        }
        set {
            self.settingsEntity.themeIDSetting = newValue
        }
    }
    
    public override init() {
        super.init()
        
		self.load()
    }
    
    // Loads the SettingsEntity from the database
    public func load() {
        
        var settingsList = self.getAllEntities()
        if settingsList.count > 0 {
            self.settingsEntity = settingsList[0]
        }
        else {
            self.settingsEntity = self.createEntity()
            self.saveEntities()
        }
    }
    
    // Gets the current Settings and returns them in a dictionary
    public func getHandoffSettings() -> Dictionary<NSObject, AnyObject> {
    
        var dictionary = Dictionary<NSObject, AnyObject>()
        dictionary[HANDOFF_GAMEMODE] = "\(self.gameMode.rawValue)"
        dictionary[HANDOFF_PLAYER1NAME] = "\(self.player1Name)"
        dictionary[HANDOFF_PLAYER2NAME] = "\(self.player2Name)"
        dictionary[HANDOFF_PLAYER1MARK] = "\(self.player1Mark)"
        dictionary[HANDOFF_PLAYER2MARK] = "\(self.player2Mark)"
        dictionary[HANDOFF_FIRSTMOVE] = "\(self.firstMove.rawValue)"
        dictionary[HANDOFF_DIFFICULTY] = "\(self.difficulty.rawValue)"
        dictionary[HANDOFF_THEMEID_KEY] = "\(self.themeID)"
    	return dictionary
    }
    
    // Sets the current Settings from the values in the dictionary
    public func setHandoffSettings(dictionary: Dictionary<NSObject, AnyObject>) {
        self.gameMode = GameMode(rawValue: dictionary[HANDOFF_GAMEMODE]!.integerValue)!
        self.player1Name = "\(dictionary[HANDOFF_PLAYER1NAME])"
        self.player2Name = "\(dictionary[HANDOFF_PLAYER2NAME])"
        self.player1Mark = "\(dictionary[HANDOFF_PLAYER1MARK])"
        self.player2Mark = "\(dictionary[HANDOFF_PLAYER2MARK])"
        self.firstMove = FirstMove(rawValue: dictionary[HANDOFF_FIRSTMOVE]!.integerValue)!
        self.difficulty = Difficulty(rawValue: dictionary[HANDOFF_DIFFICULTY]!.integerValue)!
        self.themeID = dictionary[HANDOFF_THEMEID_KEY]!.integerValue
        self.saveEntities()
    }
    
}