//
//  SettingsEntity.swift
//  TicTacToe
//
//  Created by Kevin McNeish on 3/24/15.
//  Copyright (c) 2015 Oak Leaf Enterprises, Inc. All rights reserved.
//

import Foundation
import CoreData

public class SettingsEntity: NSManagedObject {

    @NSManaged private var gameMode: NSNumber
    @NSManaged public var player1Name: String?
    @NSManaged public var player2Name: String?
    @NSManaged var player1Mark: String?
    @NSManaged var player2Mark: String?
    @NSManaged private var firstMove: NSNumber
    @NSManaged private var difficulty: NSNumber
    @NSManaged private var themeID: NSNumber
    
    public var gameModeSetting: GameMode {
        get {
            return GameMode(rawValue: self.gameMode.integerValue)!
        }
        set {
            self.gameMode = newValue.rawValue
        }
    }
    
    public var firstMoveSetting: FirstMove {
        get {
            return FirstMove(rawValue: self.firstMove.integerValue)!
        }
        set {
            self.firstMove = newValue.rawValue
        }
    }
    
    public var difficultySetting: Difficulty {
        get {
            return Difficulty(rawValue: self.difficulty.integerValue)!
        }
        set {
            self.difficulty = newValue.rawValue
        }
    }
    
    public var player1NameSetting: String {
        get {
            if String.isNilOrEmpty(self.player1Name) {
                return NSLocalizedString("You", comment: "First person reference to the player")            }
            else {
				return self.player1Name!
            }
        }
        set {
            self.player1Name = newValue
        }
    }
    
    public var player2NameSetting: String {
        get {
            
            if self.gameModeSetting == .OnePlayer {
        		return "Player 1"
            }
            else if String.isNilOrEmpty(self.player2Name) {
                return "Player 2"
            }
            else {
                return self.player2Name!
            }
        }
        set {
            self.player2Name = newValue
        }
    }
    
    public var themeIDSetting: Int {
        get {
            return self.themeID.integerValue
        }
        set {
            self.themeID = newValue
        }
    }

}
