//
//  Theme.swift
//  TicTacToe
//
//  Created by Kevin McNeish on 4/18/15.
//  Copyright (c) 2015 Oak Leaf Enterprises, Inc. All rights reserved.
//

import Foundation

public class Theme<T:ThemeEntity>: ABusinessObject<T> {
    
    let sortDescriptor = NSSortDescriptor(key: "displayOrder", ascending: true)
    
    // Returns all free themes
    public func getFreeThemes() -> Array<T> {
        
        return self.getThemesByThemeBundleID("com.oakleafsd.TicTacToe.FreeTheme")
    }
    
    // Gets all themes sorted by display order
    public func getThemesByDisplayOrder() -> Array<T> {
        
        return self.getAllEntitiesSortedBy(sortDescriptor)
    }
    
    // Get the specified theme
    public func getThemeByID(themeID: Int) -> T {
        
        let predicate = NSPredicate(format: "themeID == %d",themeID)
        
        let entityList = self.getEntitiesMatchingPredicate(predicate)
        
        return entityList[0]
    }
    
    // Get themes by bundle ID
    public func getThemesByThemeBundleID(themeBundleID: String) -> Array<T> {
        
        let predicate = NSPredicate(format: "themeBundleID == %@", themeBundleID)
        return self.getEntities(sortedBy: sortDescriptor, matchingPredicate: predicate)
    }
    
    // Get all Apple Watch themes (the user chooses)
    public func getAppleWatchThemes() -> Array<T> {
        let predicate = NSPredicate(format: "useOnAppleWatch == YES")
        return self.getEntities(sortedBy: sortDescriptor, matchingPredicate: predicate)
    }
    
    public override init() {
        super.init()
    }
}