//
//  TicTacToeEngine.swift
//  TicTacToe
//
//  Created by Kevin McNeish on 3/15/15.
//  Copyright (c) 2015 Oak Leaf Enterprises, Inc. All rights reserved.
//

import Foundation

public var globalThemeEntity: ThemeEntity!
public var globalTheme = Theme()
public let HANDOFF_PURCHASE_THEMES = "com.oakleafsd.TicTacToe.purchaseThemes"
public let HANDOFF_GAME = "com.oakleafsd.TicTacToe.handoffGame"
public let HANDOFF_DICTIONARY_MOVES_KEY = "MOVES"
public let HANDOFF_GAMEMODE = "GAMEMODE"
public let HANDOFF_PLAYER1NAME = "PLAYER1NAME"
public let HANDOFF_PLAYER2NAME = "PLAYER2NAME"
public let HANDOFF_PLAYER1MARK = "PLAYER1MARK"
public let HANDOFF_PLAYER2MARK = "PLAYER2MARK"
public let HANDOFF_FIRSTMOVE = "FIRSTMOVE"
public let HANDOFF_DIFFICULTY = "DIFFICULTY"
public let HANDOFF_THEMEID_KEY = "THEMEID"
public let CORNER_POSITIONS = [Position.a, Position.c, Position.g, Position.i]

public protocol TicTacToeDelegate {
    
    func playerMoved(player: String, position: Position, win: Array<Position>?, gameOver: Bool)
}

public enum FirstMove: Int {
    case X, O, XO
}

enum PlayerMark:  Int {
    case X, O
}

public enum Difficulty: Int {
    case Easy, Medium, Difficult
}

public enum GameMode: Int {
    case OnePlayer, TwoPlayers, Online
}

public enum Matrix: Int {
    case
    	D1, // Diagonal 1
    	H1, // Horizontal 1
    	H2, // Horizontal 2
    	H3, // Horizontal 3
    	V1, // Vertical 1
    	V2, // Vertical 2
    	V3, // Vertical 3
    	D2  // Diagonal 2
}

private let singletonInstance = TicTacToe()

public class TicTacToe {
    
    let matrix: [Matrix: Array<Position>] =
    	[
            .D1: [.a, .e, .i],
            .H1: [.a, .b, .c],
            .H2: [.d, .e, .f],
            .H3: [.g, .h, .i],
            .V1: [.a, .d, .g],
            .V2: [.b, .e, .h],
            .V3: [.c, .f, .i],
            .D2: [.c, .e, .g]
    	]
    
    func numberOfMovesInMatrixRow(matrixRow: Matrix) -> Int {
        var moveCount = 0
        let positions = self.matrix[matrixRow]!
        for position in positions {
            if contains(self.moves, Move(position: position, mark: "XO")) {
                moveCount++
            }
        }
        return moveCount
    }
    
    class var sharedInstance: TicTacToe {
        return singletonInstance
    }
    
    public var delegate: TicTacToeDelegate?
    
    public var isCurrentPlayerHuman: Bool {
        if currentPlayerMark == "X" {
            return self.isPlayerXHuman
        }
        else {
            return self.isPlayerOHuman
        }
    }
    
    public var delaySeconds = 1.5

    var firstPlayerMark = ""	// "X" or "O"
    public var currentPlayerMark: String = ""	// "X" or "O"
    var currentPlayerName: String {
        if currentPlayerMark == "X" {
            if Settings.sharedInstance.player1Mark == "X" {
                return Settings.sharedInstance.player1Name
            }
            else {
                return Settings.sharedInstance.player2Name
            }
        }
        else {
            if Settings.sharedInstance.player1Mark == "O" {
                return Settings.sharedInstance.player1Name
            }
            else {
                return Settings.sharedInstance.player2Name
            }
        }
    }
    
    public var player1Wins = 0		// Number of wins
    public var player2Wins = 0		// Number of wins
    public var ties = 0			// Number of ties

    public var isPlayerXHuman = false
    public var isPlayerOHuman = false
    var isGameOver = false
    
    var gameMatrix = Array<Int>()
    var positions = Dictionary<Position, Array<Int>>()
    
    let wins =
    [[Position.a, Position.e, Position.i],
        [Position.a, Position.b, Position.c],
        [Position.d, Position.e, Position.f],
        [Position.g, Position.h, Position.i],
        [Position.a, Position.d, Position.g],
        [Position.b, Position.e, Position.h],
        [Position.c, Position.f, Position.i],
        [Position.c, Position.e, Position.g]]
    
    // Array of corner positions. Corners get removed from this array
    // when users occupy the corresponding spot
    var cornerPositions = CORNER_POSITIONS
    
    // Array of edge positions
    let edgePositions = [Position.b, Position.d, Position.f, Position.h]
    
    // Center position
    let centerPosition = Position.e
    
    // An array of moves in the current game
    // TODO: Change this to array of Move objects
    public var moves = Array<Move>()
    
    // Tally of 2, 1, 0, -1, and -2 positions
    let PLUS_TWO = 0
    let MINUS_TWO = 1
    let PLUS_ONE = 0
    let MINUS_ONE = 1
    
    var onePositions2D = [Array<Int>(),Array<Int>()]
    var twoPositions2D = [Array<Int>(),Array<Int>()]
    
    var twoPositions = Array<Int>()
    var onePositions = Array<Int>()
    var minusOnePositions = Array<Int>()
    var minusTwoPositions = Array<Int>()
    
    var player1FirstMoveCorner = false
    var player2FirstMoveCenter = false
    var player1SecondMoveOppositeCorner = false
	public var winnerName = ""
    public var winnerMessage = ""
    
    public init() {
        
    }

    
    public func setupNewGame() {
        
        self.gameMatrix = [0,0,0,0,0,0,0,0]
        self.positions[Position.a] = [1,1,0,0,1,0,0,0]
        self.positions[Position.b] = [0,1,0,0,0,1,0,0]
        self.positions[Position.c] = [0,1,0,0,0,0,1,1]
        self.positions[Position.d] = [0,0,1,0,1,0,0,0]
        self.positions[Position.e] = [1,0,1,0,0,1,0,1]
        self.positions[Position.f] = [0,0,1,0,0,0,1,0]
        self.positions[Position.g] = [0,0,0,1,1,0,0,1]
        self.positions[Position.h] = [0,0,0,1,0,1,0,0]
        self.positions[Position.i] = [1,0,0,1,0,0,1,0]
        
        self.cornerPositions = [Position.a, Position.c, Position.g, Position.i]
        
        self.isGameOver = false
        self.moves = Array<Move>()
        
        self.player1FirstMoveCorner = false
        self.player1SecondMoveOppositeCorner = false
        self.player2FirstMoveCenter = false
        
        // Set the current player based on firstMove settings
        switch (Settings.sharedInstance.firstMove) {
        case .O:
            self.currentPlayerMark = "O"
        case .X:
            self.currentPlayerMark = "X"	
        case .XO:
            if self.currentPlayerMark == "" {
                self.currentPlayerMark = "X"
            }
            else {
                self.currentPlayerMark = (self.firstPlayerMark == "X") ? "O" : "X"
            }
        }
        self.firstPlayerMark = self.currentPlayerMark
        
        // Determine if Player O and Player X are human
        switch Settings.sharedInstance.gameMode {
        case .OnePlayer:
            self.isPlayerXHuman = (Settings.sharedInstance.player1Mark == "X")
            self.isPlayerOHuman = (Settings.sharedInstance.player1Mark == "O")
        case .TwoPlayers:
            fallthrough
        case .Online:
            self.isPlayerXHuman = true
            self.isPlayerOHuman = true
        }
    }
    
    func blockPotentialForks(otherPlayerMark: String,
        cornerMatrices: [Matrix],
        forkPositions2D: [[Position]],
        forkMatrices: [Matrix],
        blockPositions2D: [[[Position]]]) -> Position? {
            
        var blockPosition: Position? = nil
        
        for (cornerMatrixIndex, cornerMatrix) in enumerate(cornerMatrices) {
            
            // Check if there are any other moves in this row
            if self.numberOfMovesInMatrixRow(cornerMatrix) == 1 {
                
                // No other moves! check potential fork positions
                var forkPositions = forkPositions2D[cornerMatrixIndex]
                    
                for (forkPositionIndex, forkPosition) in enumerate(forkPositions) {
                    
                    // Has other user occupied this fork position?
                    if let i = find(self.moves, Move(position: forkPosition, mark: otherPlayerMark)) {
                        
                        // Yes. Check if this is the only move in the row
                        var forkMatrix = forkMatrices[cornerMatrixIndex]
                        
                        if self.numberOfMovesInMatrixRow(forkMatrix) == 1 {
                            // It is...it's a potential fork!
                            let blockPositions = blockPositions2D[cornerMatrixIndex][forkPositionIndex]
                            
                            for index in 0...blockPositions.count {
                                // Get a random block position
                                let randomIndex = Int(arc4random_uniform(UInt32(blockPositions.count)))
                                blockPosition = blockPositions[randomIndex]
                                
                                // Check if position is occupied
                                if let i = find(self.moves, Move(position: blockPosition!, mark: "XO")) {
                                    // It is, keep looking!
                                    continue
                                }
                                break
                            }
                            break
                        }
                    }
                }
            }
        }
        
        return blockPosition
    }
    
    // Moves player X to the specified position
    // 1. Adds the position matrix to the game matrix
    // 2. Removes the position from the positions array
    // 3. If a corner, removes position from the corners array
    // 4. Adds the move to the moves array
    // 5. Changes the currentPlayer setting to player O
    // 6. Sets the gameOver flag true if moves.count == 9
    func playerMove(player: String, position: Position) -> (player: String, position: Position, win: Array<Position>?, gameOver: Bool) {
        
        var playerWins = false
        var winIndex: Int?
        
        // Get the position matrix
        let positionMatrix = self.positions[position]!
        
        if player == "X" {
            // Add the position matrix to the game matrix
            for index in 0..<positionMatrix.count {
                
                self.gameMatrix[index] =
                    self.gameMatrix[index] + positionMatrix[index]
                if self.gameMatrix[index] == 3 {
                    // Player wins!
                    playerWins = true

                    self.isGameOver = true
                    winIndex = index
                }
            }
        }
        else {
            // Subtract the position matrix from the game matrix
            for index in 0..<positionMatrix.count {
                
                self.gameMatrix[index] =
                    self.gameMatrix[index] - positionMatrix[index]
                if self.gameMatrix[index] == -3 {
                    // Player wins!
                    playerWins = true
                    self.isGameOver = true
                    winIndex = index
                }
            }
        }
        
        // Remove the position from the possible choices
        self.positions.removeValueForKey(position)
        
        // If a corner, remove from list of corners
        if let itemNumber = find(self.cornerPositions, position) {
            self.cornerPositions.removeAtIndex(itemNumber)
            if self.moves.count == 0 {
                self.player1FirstMoveCorner = true
            }
        }
        
        var results = Array<(code: Int, message: String)>()
        
        results += [(code: 1, message: "Invalid Date")]
        results.append(code: 1, message: "Invalid Date")
        
        // Add to the set of moves in this game
        // NOTE: Had to store in a var first...seems like a Swift tuple bug
        let mark = self.currentPlayerMark
        self.moves.append(Move(position: position, mark: mark))
        
        // Check if game is over
        self.isGameOver = (moves.count == 9) || playerWins
        
        if self.isGameOver {
            // Check for a tie
            if moves.count == 9 && !playerWins {
                self.winnerName = NSLocalizedString("It's a tie!", comment:"A tie game")
                self.winnerMessage = ""
                self.ties++
            }
            else {
                if self.currentPlayerMark == "X" && Settings.sharedInstance.player1Mark == "X" {
                    self.player1Wins++
                    let winText = Settings.sharedInstance.player1Name.lowercaseString == "you"
                        ? NSLocalizedString("Win!", comment: "Singular win")
                        : NSLocalizedString("Wins!", comment: "Plural win")
                    
                    self.winnerName = Settings.sharedInstance.player1Name + " " + winText
                    
                    self.winnerMessage = ""
                    let streakMessage = NSLocalizedString("game streak!", comment: "winning streak")
                }
                else {
                    self.player2Wins++
                    self.winnerName = Settings.sharedInstance.player2Name + " Wins!"
                }
                
            }
        }
        
        // Save Member settings
        Member.sharedInstance.saveEntities()
        
        // Switch the current player
        self.currentPlayerMark = (player == "X") ? "O": "X"
        
        return (player, position, (playerWins ? wins[winIndex!] : nil), self.isGameOver)
    }
    
    public func currentPlayerMove(position: Position) -> (player: String, position: Position, win: Array<Position>?, gameOver: Bool) {
        
        return playerMove(self.currentPlayerMark, position: position)
    }
    
    // Checks the positions array to see if the
    // specified position is open
    public func isPositionOpen(position: Position) -> Bool {
        return self.positions[position] != nil
    }
    
    func collectPositionsIntoArrays() {
        
        self.twoPositions2D = [Array<Int>(),Array<Int>()]
        self.onePositions2D = [Array<Int>(),Array<Int>()]
        
        for index in 0..<self.gameMatrix.count {
            
            var positionValue = gameMatrix[index]
            
            switch positionValue {
            case 2:
                self.twoPositions2D[PLUS_TWO].append(index)
            case 1:
                self.onePositions2D[PLUS_ONE].append(index)
            case -1:
                self.onePositions2D[MINUS_ONE].append(index)
            case -2:
                self.twoPositions2D[MINUS_TWO].append(index)
            default:
                break
            }
        }
    }
    
    func printlnGameMatrix() {
        
        var matrix = ""
        
        for position in gameMatrix {
            matrix += "\(position) "
        }
        
        //println(matrix)
    }
    
}