//
//  ToDo.swift
//  ToDoListDemo
//
//  Created by Kevin McNeish on 3/20/15.
//  Copyright (c) 2015 Oak Leaf Enterprises, Inc. All rights reserved.
//

import Foundation

class ToDo<T: ToDoEntity>: ABusinessObject<T> {
    
    var entityList: Array<T>!
    
    override init() {
        super.init()
    }
    
    // Get all entities sorted by display order
    func getAllEntitiesByDisplayOrder() -> Array<T> {
    
        let sortDescriptor = NSSortDescriptor(key: "displayOrder", ascending: true)
        self.entityList = self.getAllEntitiesSortedBy(sortDescriptor)
        return self.entityList!
    }
    
    // Move the object at the source index path to the destination index path
    func moveObjectAtIndexPath(sourceIndexPath: NSIndexPath, destinationIndexPath: NSIndexPath) {
        
        let from = sourceIndexPath.row
        let to = destinationIndexPath.row
        
        if from == to {
            return
        }
        
        // Get the entity to be reordered, remove it from
        // its old position and add it in its new position
        let toDoEntity = self.entityList[from]
        self.entityList.removeAtIndex(from)
        self.entityList.insert(toDoEntity, atIndex: to)
        
        // Set the new order of the object
        var lower = 0.0
        var upper = 0.0
        
        // Check for an item before it
        if to > 0 {
            lower = Double(self.entityList[to-1].displayOrder)
        }
        else {
            lower = Double(self.entityList[1].displayOrder) - 2.0
        }
        
        // Check for an item after it
        if to < self.entityList.count - 1 {
            upper = Double(self.entityList[to + 1].displayOrder)
        }
        else {
            upper = Double(self.entityList[to - 1].displayOrder) + 2.0
        }
        
        // Add the upper and lower, divide by two
        // to derive the new order
        let newOrder = (lower + upper) / 2.0
        toDoEntity.displayOrder = newOrder
    }
    
    // Remove the object at the specified index path
    func removeObjectAtIndexPath(indexPath: NSIndexPath) {
        self.entityList.removeAtIndex(indexPath.row)
    }
    
    // Create a new entity with the specified description, add it to the list
    // and return the new entity to the caller
    func addItemToList(desc: String) -> ToDoEntity {
        var toDoEntity = self.createEntity()
        toDoEntity.desc = desc
        toDoEntity.displayOrder = self.entityList.count + 1
        self.saveEntities()
        self.entityList.append(toDoEntity)
        return toDoEntity
    }
    
    // Delete the specified entity
    override func deleteEntity(entity: T) {
        super.deleteEntity(entity)
        self.saveEntities()
    }
}