//
//  ViewController.swift
//  WebViewDemo
//
//  Created by Kevin McNeish on 5/24/15.
//  Copyright (c) 2015 Oak Leaf Enterprises, Inc. All rights reserved.
//

import UIKit

class ViewController: UIViewController, UIWebViewDelegate {

    @IBOutlet weak var webView: UIWebView!
    @IBOutlet weak var textField: UITextField!
    @IBOutlet weak var activityIndicator: UIActivityIndicatorView!
    @IBOutlet weak var backButton: UIBarButtonItem!
    @IBOutlet weak var forwardButton: UIBarButtonItem!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        // Do any additional setup after loading the view, typically from a nib.
        self.textField.autoresizingMask = UIViewAutoresizing.FlexibleWidth

        self.setButtonsEnabled()
        self.gotoURL("http://www.apple.com")
    }
    
    func gotoURL(url: String) {
        self.activityIndicator.startAnimating()
        if let url = NSURL(string: url) {
            
            let urlRequest = NSURLRequest(URL: url)
            self.webView.loadRequest(urlRequest)
        }
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    func webViewDidFinishLoad(webView: UIWebView) {
        self.activityIndicator.stopAnimating()
        self.setButtonsEnabled()
    }
    
    func setButtonsEnabled() {
        if (self.webView.canGoBack) {
            self.backButton.enabled = true;
        }
        else {
            self.backButton.enabled = false;
        }
        if (self.webView.canGoForward) {
            self.forwardButton.enabled = true;
        }
        else {
            self.forwardButton.enabled = false;
        }
    }

    @IBAction func urlEntered(sender: AnyObject) {
        let urlString = self.textField.text.lowercaseString
        if !urlString.hasPrefix("http://") {
            self.textField.text = "http://" + self.textField.text
        }
        self.gotoURL(self.textField.text)
    }
    
    @IBAction func goBack(sender: AnyObject) {
        self.webView.goBack()
    }

    @IBAction func goForward(sender: AnyObject) {
        self.webView.goForward()
    }
}

