//
//  ShipmentViewController.swift
//  iDeliverMobileCD
//
//  Created by Kevin McNeish on 12/30/14.
//  Copyright (c) 2014 Oak Leaf Enterprises, Inc. All rights reserved.
//

import UIKit

enum ShipmentSection: Int {
    
    case Name, Address, Phone, Text, DeliveryStatus, ID
    static let allValues = [Name, Address, Phone, Text, DeliveryStatus, ID]

}

class ShipmentViewController: UITableViewController, DeliveryStatusDelegate {
    
    var shipment: Shipment<ShipmentEntity>!
    var shipmentEntity: ShipmentEntity!
    var shipmentItem = ShipmentItem()
    var shipmentItemList = Array<ShipmentItemEntity>()

    override func viewDidLoad() {
        super.viewDidLoad()

        self.shipmentItemList = self.shipmentItem.getShipmentItemsForShipmentID(self.shipmentEntity.shipmentID)
        
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }

    // MARK: - Table view data source

    override func numberOfSectionsInTableView(tableView: UITableView) -> Int {
        // Return the number of sections.
        return 2
    }

    override func tableView(tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        
        // Return the number of rows in the section.

        var rowCount = 0
        
        if section == 0 {
            rowCount = ShipmentSection.allValues.count
        }
        else if section == 1
        {
            rowCount = shipmentItemList.count
        }
        return rowCount
    }

    override func tableView(tableView: UITableView, cellForRowAtIndexPath indexPath: NSIndexPath) -> UITableViewCell {
        
        var cell: UITableViewCell!
        
        if indexPath.section == 0 {
            
            switch indexPath.row {
                
            case ShipmentSection.Name.rawValue:
                cell = tableView.dequeueReusableCellWithIdentifier("NameCell", forIndexPath: indexPath) as! UITableViewCell
                cell.textLabel?.text = self.shipmentEntity.name
            
            case ShipmentSection.Address.rawValue:
                cell = tableView.dequeueReusableCellWithIdentifier("AddressCell", forIndexPath: indexPath) as! UITableViewCell
                cell.textLabel?.text = self.shipmentEntity.address
                cell.detailTextLabel?.text = self.shipmentEntity.city + ", " +
                	self.shipmentEntity.region + " " +
                	self.shipmentEntity.postalCode
            
            case ShipmentSection.Phone.rawValue:
                cell = tableView.dequeueReusableCellWithIdentifier("PhoneCell", forIndexPath: indexPath) as! UITableViewCell
                cell.detailTextLabel?.text = self.shipmentEntity.phone
            
            case ShipmentSection.Text.rawValue:
                cell = tableView.dequeueReusableCellWithIdentifier("TextCell", forIndexPath: indexPath) as! UITableViewCell
                cell.detailTextLabel?.text = self.shipmentEntity.text
            
            case ShipmentSection.DeliveryStatus.rawValue:
                cell = tableView.dequeueReusableCellWithIdentifier("DeliveryStatusCell", forIndexPath: indexPath) as! UITableViewCell
                cell.detailTextLabel?.text = self.shipmentEntity.deliveryStatus.valueForKey("statusDescription") as? String
            
            case ShipmentSection.ID.rawValue:
                cell = tableView.dequeueReusableCellWithIdentifier("ShipmentIDCell", forIndexPath: indexPath) as! UITableViewCell
                cell.detailTextLabel?.text = self.shipmentEntity.shipmentID
            
            default:
                break
            }
        }
        else if indexPath.section == 1 {
            let shipItem = self.shipmentItemList[indexPath.row]
            
            cell = tableView.dequeueReusableCellWithIdentifier("ShipmentItemCell", forIndexPath: indexPath) as! UITableViewCell
            
            cell.textLabel?.text = "(\(shipItem.quantity)) \(shipItem.itemDescription)"
            
            if shipItem.itemDescription.rangeOfString("TV") != nil {
             	cell.imageView?.image = UIImage(named: "AppleTVCellImage")
            }
        }
        
        return cell
    }

    override func prepareForSegue(segue: UIStoryboardSegue, sender: AnyObject?) {
        
        if segue.identifier == "DeliveryStatusSegue" {
            var tvc = segue.destinationViewController as! DeliveryStatusViewController
            tvc.shipmentEntity = self.shipmentEntity
            tvc.delegate = self
        }
    }
    
    func updateDeliveryStatus(deliveryStatus: DeliveryStatusEntity) {
        // Change the delivery status and save changes
        self.shipmentEntity.deliveryStatus = deliveryStatus
        self.shipment.saveEntities()
        
        // Update the delivery status cell
        self.tableView.reloadRowsAtIndexPaths([NSIndexPath(forRow: 4, inSection: 0)], withRowAnimation: UITableViewRowAnimation.None)
    }
}
